library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.simeck_pkg.all;
---------------------------------------------

entity dp is
port (
		reset, 
		clk              : in std_logic;
                mode             : in std_logic;
        data_block       : in block_elem;
		cont_irounds     : in unsigned(reg_rounds-1 downto 0 );
		cont_steps       : in unsigned(reg_steps -1 downto 0 ); 
                message_in       : in std_logic_vector(31 downto 0 );
                message_in2      : in std_logic_vector(2 downto 0);
                message_out      : out std_logic_vector(31 downto 0);
        encoding_message : out block_elem
);
end entity;

architecture main of dp is 

signal message_block  :  block_elem;

signal message_block_tmp : block_elem;

--signal cont_irounds   :  unsigned(reg_rounds-1 downto 0 );
--signal cont_steps     :  unsigned(reg_steps -1 downto 0 ); 

signal round_message_o : word_elem;
signal round_message_i1,
       round_message_i2,
       round_message_i3 : word_elem;

signal round_message_oo : word_elem;
signal round_message_ii1,
       round_message_ii2,
       round_message_ii3 : word_elem;
	   
signal d_message_block0,
       d_message_block1,
       d_message_block2,	   
	   d_message_block3,
	   d_message_block4,
	   d_message_block5,
	   d_message_block6,
	   d_message_block7   : word_elem;

signal message_block2_back,
       message_block3_back,
       message_block6_back,
       message_block7_back 	   : word_elem;
	   
signal message_block0_feedback,
       message_block1_feedback,
	   message_block4_feedback,
	   message_block5_feedback  : word_elem;
 
	   

signal rc1_tmp, rc2_tmp  : std_logic;
signal RC1_ext_tmp, RC2_ext_tmp  : std_logic_vector(5 downto 0);
signal lfsr_tmp          : std_logic_vector(5 downto 0);
signal lfsr_tmp_out      : std_logic_vector(5 downto 0);

begin


------------------------------------------------
--constants
 process
   begin
    wait until rising_edge(clk);
    if reset ='1' then
      lfsr_tmp <= "111111";
    else
      lfsr_tmp <= lfsr_tmp_out;
    end if;
  end process;  
	  
lfsr_para2_1:  entity work.lfsr_para2
port map
( lfsr_in   =>  lfsr_tmp
  ,lfsr_out  =>  lfsr_tmp_out
  ,rc1       =>  rc1_tmp
  ,rc2       =>  rc2_tmp
  ,RC1_ext       =>  RC1_ext_tmp
  ,RC2_ext       =>  RC2_ext_tmp
  );
-----------------------------------------------
roundfun1 :  entity work.roundfun
port map
(  a_i   => round_message_i1
   ,b_i   => round_message_i2
   ,c_i   => round_message_i3
   ,d_o   => round_message_o
   ); 

roundfun2 :  entity work.roundfun
	port map
(  a_i   => round_message_ii1
   ,b_i   => round_message_ii2
   ,c_i   => round_message_ii3
   ,d_o   => round_message_oo
   );
 -------------------------------------------------------------     
 round_message_i1 <= message_block(2);
 round_message_i2 <= message_block(3);
 round_message_i3 <= cont_round (word_sz-2 downto 0) & rc1_tmp;
 round_message_ii1 <= message_block(6);
 round_message_ii2 <= message_block(7);
 round_message_ii3 <= cont_round (word_sz-2 downto 0) & rc2_tmp;  
 
 --------------------------------------------------------------
 d_message_block0     <= message_block2_back;
 d_message_block1     <= message_block3_back; 
 
 d_message_block2     <= round_message_o when cont_irounds < 5 else
                         message_block4_feedback;
 d_message_block3     <= message_block (2) when cont_irounds < 5 else 
                         message_block5_feedback ; 

 d_message_block4     <= message_block6_back;
 d_message_block5     <= message_block7_back; 
				 						   
 d_message_block6     <= round_message_oo when cont_irounds < 5 else
                         message_block0_feedback;
 d_message_block7     <= message_block (6) when cont_irounds < 5 else 
                         message_block1_feedback ; 			 
				 
 message_block4_feedback  <=  message_block (4) xor round_message_oo xor cont_round(word_sz-1 downto 0); 
 message_block5_feedback  <=  message_block (5) xor message_block (6) xor (cont_round(word_sz -9 downto 0)& "00" & RC2_ext_tmp);	  
 
 
 message_block0_feedback  <=  message_block (0) xor round_message_o xor cont_round(word_sz-1 downto 0); 
 message_block1_feedback  <=  message_block (1) xor message_block (2) xor (cont_round(word_sz -9 downto 0)& "00" & RC1_ext_tmp);	  
 
--------------------------------------------------------	  
process 
 begin
   wait until rising_edge (clk);
   if cont_irounds =0  then
       message_block2_back <= message_block(2);
	   message_block3_back <= message_block(3);
	   message_block6_back <= message_block(6);
	   message_block7_back <= message_block(7);
  end if;
 
end process;


---------------------------------------
 -------------------------------------
  process
  begin
   wait until rising_edge (clk);
    if reset ='1' and mode='0'  then
           message_block  <= data_block;
    elsif mode ='0' then  
           if cont_irounds = 5 and cont_steps < 18 then
       message_block (0) <= d_message_block0;
           message_block (1) <= d_message_block1;

           message_block (4) <= d_message_block4;
       message_block (5) <= d_message_block5;
           end if;

           if cont_steps < 18 then
       message_block (2) <= d_message_block2;
       message_block (3) <= d_message_block3;

           message_block (6) <= d_message_block6;
       message_block (7) <= d_message_block7;
           end if;
   elsif reset ='1' and mode ='1' then
            message_block(0) <= message_block_tmp(0) xor message_in(23 downto 0);
            message_block(1)(7 downto 0) <= message_block_tmp(1)(7 downto 0) xor message_in(31 downto 24); 
            message_block(7)(2 downto 0 ) <= message_block_tmp(7)(2 downto 0) xor message_in2(2 downto 0);   
   elsif  mode='1' then
               if cont_irounds = 5 and cont_steps < 18 then
       message_block (0) <= d_message_block0;
           message_block (1) <= d_message_block1;

           message_block (4) <= d_message_block4;
       message_block (5) <= d_message_block5;
           end if;

           if cont_steps < 18 then
       message_block (2) <= d_message_block2;
       message_block (3) <= d_message_block3;

           message_block (6) <= d_message_block6;
       message_block (7) <= d_message_block7;
           end if;
 
     

   end if;

  end process;






    message_out(23 downto 0) <= message_block(0) xor message_in(23 downto 0 );
    message_out(31 downto 24) <= message_block(0)(7 downto 0) xor message_in(31 downto 24 );
       
    message_block_tmp <= message_block;
    	  
  	encoding_message <= message_block;

	
end architecture;





